import Mixin from '@ember/object/mixin'
import { run } from '@ember/runloop'
import { get } from '@ember/object'
import { A } from '@ember/array'
import { isEmpty } from '@ember/utils'

import KeyMap from 'en-select/utils/keymap'

export default Mixin.create({
  selectableOptions: A([]),
  focusedOption: null,

  focusOnFirstOption() {
    let options = get(this, 'selectableOptions')
    let focused = get(this, 'focused')

    /*
     * If the focus goes out of range, we focus
     * on the first option automatically.
     */

    if (options && !options.includes(focused)) {
      run.scheduleOnce('afterRender', this, () => {
        this.focusOption(options[0])
      })
    }
  },

  scrollIfFocusNotVisible() {
    let node = this.element
    if (!node || !(node instanceof HTMLElement)) return

    let focusedOption = this.element.querySelector('.is-focused')
    if (focusedOption) focusedOption.scrollIntoView()
  },

  keyUp(event) {
    const { which, keyCode } = event
    const key = which || keyCode

    let isRelevant = false

    Object.keys(KeyMap).forEach(k => {
      if (KeyMap[k] === key) isRelevant = true
    })

    if (isRelevant) {
      event.preventDefault()
      event.stopPropagation()
    }

    switch (key) {
      case KeyMap.up:
        this._move('up', event)
        break

      case KeyMap.down:
        this._move('down', event)
        break

      case KeyMap.enter:
        break

      default:
        break
    }
  },

  _findIndices(options, focused) {
    let indexOfFocus = options.indexOf(focused)

    let lastIsFocused = indexOfFocus === options.length - 1
    let firstIsFocused = indexOfFocus === 0
    let nothingIsFocused = isEmpty(focused)

    return {
      indexOfFocus,
      firstIsFocused,
      lastIsFocused,
      nothingIsFocused,
    }
  },

  _findNextFocus(direction, options, focused) {
    let nextFocus

    let {
      indexOfFocus,
      firstIsFocused,
      lastIsFocused,
      nothingIsFocused,
    } = this._findIndices(options, focused)

    let up = direction === 'up'
    let down = direction === 'down'

    /*
     * In case the first element is focused, and the direction is
     * up, we move to the last element.
    */

    if ((firstIsFocused || nothingIsFocused) && up) {
      nextFocus = options[options.length - 1]
    } else if ((lastIsFocused || nothingIsFocused) && down) {
      nextFocus = options[0]
    } else {
      let delta

      if (direction === 'up') {
        delta = -1
      } else if (direction === 'down') {
        delta = 1
      }

      nextFocus = options[indexOfFocus + delta]
    }

    return nextFocus
  },

  _move(direction) {
    let options = get(this, 'selectableOptions')
    let focused = get(this, 'focused')

    let nextFocus = this._findNextFocus(direction, options, focused)

    if (isEmpty(nextFocus)) {
      return
    }

    this.focusOption(nextFocus)
    this.scrollIfFocusNotVisible()
  }
})
